@@ -27,7 +27,7 @@
 #' @return A [arrow::Table][Table], or a `data.frame` if `as_data_frame` is
 #' `TRUE` (the default).
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' tf <- tempfile()
 #' on.exit(unlink(tf))
 #' write_parquet(mtcars, tf)
@@ -52,10 +52,16 @@ read_parquet <- function(file,
     schema <- reader$GetSchema()
     names <- names(schema)
     indices <- match(vars_select(names, !!col_select), names) - 1L
-    tab <- reader$ReadTable(indices)
+    tab <- tryCatch(
+      reader$ReadTable(indices),
+      error = read_compressed_error
+    )
   } else {
     # read all columns
-    tab <- reader$ReadTable()
+    tab <- tryCatch(
+      reader$ReadTable(),
+      error = read_compressed_error
+    )
   }
 
   if (as_data_frame) {
@@ -122,7 +128,7 @@ read_parquet <- function(file,
 #' @return the input `x` invisibly.
 #'
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' tf1 <- tempfile(fileext = ".parquet")
 #' write_parquet(data.frame(x = 1:5), tf1)
 #'
@@ -449,7 +455,7 @@ ParquetFileWriter$create <- function(schema,
 #'
 #' @export
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' f <- system.file("v0.7.1.parquet", package="arrow")
 #' pq <- ParquetFileReader$create(f)
 #' pq$GetSchema()
