@@ -65,6 +65,11 @@
 #'    coerced to an R vector before taking.
 #' - `$Filter(i, keep_na = TRUE)`: return an `Table` with rows at positions where logical
 #'    vector or Arrow boolean-type `(Chunked)Array` `i` is `TRUE`.
+#' - `$SortIndices(names, descending = FALSE)`: return an `Array` of integer row
+#'    positions that can be used to rearrange the `Table` in ascending or descending
+#'    order by the first named column, breaking ties with further named columns.
+#'    `descending` can be a logical vector of length one or of the same length as
+#'    `names`.
 #' - `$serialize(output_stream, ...)`: Write the table to the given
 #'    [OutputStream]
 #' - `$cast(target_schema, safe = TRUE, options = cast_options(safe))`: Alter
@@ -91,11 +96,9 @@
 #' as.data.frame(tab[4:8, c("gear", "hp", "wt")])
 #' }
 #' @export
-Table <- R6Class("Table", inherit = ArrowObject,
+Table <- R6Class("Table", inherit = ArrowTabular,
   public = list(
-    column = function(i) {
-      Table__column(self, i)
-    },
+    column = function(i) Table__column(self, i),
     ColumnNames = function() Table__ColumnNames(self),
     RenameColumns = function(value) Table__RenameColumns(self, value),
     GetColumnByName = function(name) {
@@ -107,21 +110,16 @@ Table <- R6Class("Table", inherit = ArrowObject,
     AddColumn = function(i, new_field, value) Table__AddColumn(self, i, new_field, value),
     SetColumn = function(i, new_field, value) Table__SetColumn(self, i, new_field, value),
     field = function(i) Table__field(self, i),
-
     serialize = function(output_stream, ...) write_table(self, output_stream, ...),
-    ToString = function() ToString_tabular(self),
-
-    cast = function(target_schema, safe = TRUE, options = cast_options(safe)) {
+    to_data_frame = function() {
+      Table__to_dataframe(self, use_threads = option_use_threads())
+    },
+    cast = function(target_schema, safe = TRUE, ..., options = cast_options(safe, ...)) {
       assert_is(target_schema, "Schema")
-      assert_is(options, "CastOptions")
       assert_that(identical(self$schema$names, target_schema$names), msg = "incompatible schemas")
       Table__cast(self, target_schema, options)
     },
-
-    SelectColumns = function(indices) {
-      Table__SelectColumns(self, indices)
-    },
-
+    SelectColumns = function(indices) Table__SelectColumns(self, indices),
     Slice = function(offset, length = NULL) {
       if (is.null(length)) {
         Table__Slice1(self, offset)
@@ -129,39 +127,16 @@ Table <- R6Class("Table", inherit = ArrowObject,
         Table__Slice2(self, offset, length)
       }
     },
-    Take = function(i) {
-      if (is.numeric(i)) {
-        i <- as.integer(i)
-      }
-      if (is.integer(i)) {
-        i <- Array$create(i)
-      }
-      call_function("take", self, i)
-    },
-    Filter = function(i, keep_na = TRUE) {
-      if (is.logical(i)) {
-        i <- Array$create(i)
-      }
-      call_function("filter", self, i, options = list(keep_na = keep_na))
-    },
-
+    # Take, Filter, and SortIndices are methods on ArrowTabular
     Equals = function(other, check_metadata = FALSE, ...) {
       inherits(other, "Table") && Table__Equals(self, other, isTRUE(check_metadata))
     },
-
-    Validate = function() {
-      Table__Validate(self)
-    },
-
-    ValidateFull = function() {
-      Table__ValidateFull(self)
-    },
-
+    Validate = function() Table__Validate(self),
+    ValidateFull = function() Table__ValidateFull(self),
     invalidate = function() {
       .Call(`_arrow_Table__Reset`, self)
       super$invalidate()
     }
-
   ),
 
   active = list(
@@ -186,54 +161,6 @@ Table <- R6Class("Table", inherit = ArrowObject,
   )
 )
 
-arrow_attributes <- function(x, only_top_level = FALSE) {
-  att <- attributes(x)
-
-  removed_attributes <- character()
-  if (identical(class(x), c("tbl_df", "tbl", "data.frame"))) {
-    removed_attributes <- c("class", "row.names", "names")
-  } else if (inherits(x, "data.frame")) {
-    removed_attributes <- c("row.names", "names")
-  } else if (inherits(x, "factor")) {
-    removed_attributes <- c("class", "levels")
-  } else if (inherits(x, "integer64") || inherits(x, "Date")) {
-    removed_attributes <- c("class")
-  } else if (inherits(x, "POSIXct")) {
-    removed_attributes <- c("class", "tzone")
-  } else if (inherits(x, "hms") || inherits(x, "difftime")) {
-    removed_attributes <- c("class", "units")
-  }
-
-  att <- att[setdiff(names(att), removed_attributes)]
-  if (isTRUE(only_top_level)) {
-    return(att)
-  }
-
-  if (is.data.frame(x)) {
-    columns <- map(x, arrow_attributes)
-    out <- if (length(att) || !all(map_lgl(columns, is.null))) {
-      list(attributes = att, columns = columns)
-    }
-    return(out)
-  }
-
-  columns <- NULL
-  if (is.list(x) && !inherits(x, "POSIXlt")) {
-    # for list columns, we also keep attributes of each
-    # element in columns
-    columns <- map(x, arrow_attributes)
-    if (all(map_lgl(columns, is.null))) {
-      columns <- NULL
-    }
-  }
-
-  if (length(att) || !is.null(columns)) {
-    list(attributes = att, columns = columns)
-  } else {
-    NULL
-  }
-}
-
 Table$create <- function(..., schema = NULL) {
   dots <- list2(...)
   # making sure there are always names
@@ -248,106 +175,5 @@ Table$create <- function(..., schema = NULL) {
   }
 }
 
-#' @export
-as.data.frame.Table <- function(x, row.names = NULL, optional = FALSE, ...) {
-  df <- Table__to_dataframe(x, use_threads = option_use_threads())
-  if (!is.null(r_metadata <- x$metadata$r)) {
-    df <- apply_arrow_r_metadata(df, .unserialize_arrow_r_metadata(r_metadata))
-  }
-  df
-}
-
-#' @export
-as.list.Table <- as.list.RecordBatch
-
-#' @export
-row.names.Table <- row.names.RecordBatch
-
-#' @export
-dimnames.Table <- dimnames.RecordBatch
-
-#' @export
-dim.Table <- function(x) c(x$num_rows, x$num_columns)
-
 #' @export
 names.Table <- function(x) x$ColumnNames()
-
-#' @export
-`names<-.Table` <- function(x, value) x$RenameColumns(value)
-
-#' @export
-`[.Table` <- `[.RecordBatch`
-
-#' @export
-`[[.Table` <- `[[.RecordBatch`
-
-#' @export
-`[[<-.Table` <- function(x, i, value) {
-  if (!is.character(i) & !is.numeric(i)) {
-    stop("'i' must be character or numeric, not ", class(i), call. = FALSE)
-  } else if (is.na(i)) {
-    # Catch if a NA_character or NA_integer is passed. These are caught elsewhere
-    # in cpp (i.e. _arrow_RecordBatch__column_name)
-    # TODO: figure out if catching in cpp like ^^^ is preferred
-    stop("'i' cannot be NA", call. = FALSE)
-  }
-
-  if (is.null(value)) {
-    if (is.character(i)) {
-      i <- match(i, names(x))
-    }
-    x <- x$RemoveColumn(i - 1L)
-  } else {
-    if (!is.character(i)) {
-      # get or create a/the column name
-      if (i <= x$num_columns) {
-        i <- names(x)[i]
-      } else {
-        i <- as.character(i)
-      }
-    }
-
-    # auto-magic recycling on non-ArrowObjects
-    if (!inherits(value, "ArrowObject")) {
-      value <- vctrs::vec_recycle(value, x$num_rows)
-    }
-
-    # construct the field
-    if (!inherits(value, "ChunkedArray")) {
-      value <- chunked_array(value)
-    }
-    new_field <- field(i, value$type)
-
-    if (i %in% names(x)) {
-      i <- match(i, names(x)) - 1L
-      x <- x$SetColumn(i, new_field, value)
-    } else {
-      i <- x$num_columns
-      x <- x$AddColumn(i, new_field, value)
-    }
-  }
-  x
-}
-
-#' @export
-`$<-.Table` <- function(x, i, value) {
-  assert_that(is.string(i))
-  # We need to check if `i` is in names in case it is an active binding (e.g.
-  # `metadata`, in which case we use assign to change the active binding instead
-  # of the column in the table)
-  if (i %in% ls(x)) {
-    assign(i, value, x)
-  } else {
-    x[[i]] <- value
-  }
-  x
-}
-
-#' @export
-`$.Table` <- `$.RecordBatch`
-
-#' @export
-head.Table <- head.RecordBatch
-
-#' @export
-tail.Table <- tail.RecordBatch
